// mxdwg.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"
#include "MxGeneralIo.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "mxdwg.h"

#ifndef _DVT //For DVT, we don't put extern C for past compatibility

#ifdef __cplusplus    // If used by C++ code, 
extern "C" {          // we need to export the C interface
#endif

#endif
typedef struct _internal_wdg_st
{
	PVOID pDrvHandle;
	unsigned long refreshTime;
} WDG_ST, *PWDG_ST;

#define WDG_RST_ADDRESS			0x72
#define WDG_TIMER_ADDRESS		0x73
#define SUPERIO_LOGIC_DEVICE	0x7
#define DEVICE_IO_TYPE			IO_TYPE_SUPERIO
#define	KB_RST_VALUE			0x40

/**
 * Open Watchdog
 * @param  [in] time - initial refresh time
 * @return Pointer to Watchdog handle. Return -1 on failure.
 */
PVOID mxwdg_open(unsigned long time)
{
	PWDG_ST pWdg = (PWDG_ST) malloc(sizeof(WDG_ST));
	DWORD dwUnit = 0x80; // 0x80 is second, 0 is minute

	// The range is 0~255 second and 5~255 minute
	pWdg->refreshTime = time;
	if (time>0xff)
	{
		pWdg->refreshTime /= 60;
		if (time%60)
		{
			pWdg->refreshTime ++;
		}
	}

	if (MxGeneralIoOpen())
	{
		tagSuperIoStruct SuperIoStruct;
		WORD wRSTAddr = (WORD)WDG_RST_ADDRESS; 
		WORD wTimerAddr = (WORD)WDG_TIMER_ADDRESS; 
		DWORD	dwPortVal = 0;
		BOOL	bResult;

		SuperIoStruct.bLogicDev = SUPERIO_LOGIC_DEVICE;
		SuperIoStruct.bIndexPortAddr = 0x2E;
		SuperIoStruct.bDataPortAddr = 0x2F;
		SuperIoStruct.bIndexAddr = 0x7;  //UCHAR bIndexAddr;		//0x7
		SuperIoStruct.bDelayAddr = 0xEB; //UCHAR bDelayAddr;		//0xEB	
		SuperIoStruct.bIndexPortData = 0x87;//UCHAR bIndexPortData;	//0x87
		SuperIoStruct.bDelayData = 0xAA;	//UCHAR bDelayData;		//0xAA

		/* Set KB RST */
		MxGetGeneralIoValEx(DEVICE_IO_TYPE,wRSTAddr, &dwPortVal, 1,SuperIoStruct); //printf("dwPortVal = 0x%x\n",dwPortVal);
		dwPortVal &= 0x80; // Clear unit bit (second or minute)
		dwPortVal = dwPortVal | dwUnit | (DWORD)KB_RST_VALUE; //dwPortVal = 0xC0;
		bResult = MxSetGeneralIoValEx(DEVICE_IO_TYPE,wRSTAddr,dwPortVal,1,SuperIoStruct);

		/* Set time register*/
		dwPortVal = pWdg->refreshTime;
		bResult = MxSetGeneralIoValEx(DEVICE_IO_TYPE,wTimerAddr,dwPortVal,1,SuperIoStruct);
		
		return (PVOID) pWdg;
	}

	free(pWdg);

	printf("mxwdg_open: MxGeneralIoOpen!\r\n");
	return (PVOID) INVALID_HANDLE_VALUE;
}
/**
 * Refresh the watchdog timer
 * @param  [in] fd - the handle
 * @return 0 on success, otherwise the function fails
 */
int mxwdg_refresh(PVOID fd)
{
	PWDG_ST pWdg = (PWDG_ST) fd;
	DWORD dwUnit = 0x80; // 0x80 is second, 0 is minute
	if (!fd) return -1;

	DWORD	dwPortVal = 0;
	BOOL	bResult;
	WORD	wRSTAddr = (WORD)WDG_RST_ADDRESS; 
	WORD	wTimerAddr = (WORD)WDG_TIMER_ADDRESS; 
	tagSuperIoStruct SuperIoStruct;

	/*Prepare superio parameters*/
	SuperIoStruct.bLogicDev = SUPERIO_LOGIC_DEVICE; /*COM1:logic device 1, COM2: logic device 2*/
	SuperIoStruct.bIndexPortAddr = 0x2E;
	SuperIoStruct.bDataPortAddr = 0x2F;

	/* Set KB RST */
	MxGetGeneralIoValEx(DEVICE_IO_TYPE,wRSTAddr, &dwPortVal, 1,SuperIoStruct); //printf("dwPortVal = 0x%x\n",dwPortVal);
	dwPortVal &= 0x80; // Clear unit bit (second or minute)
	dwPortVal = dwPortVal | dwUnit | (DWORD)KB_RST_VALUE; //dwPortVal = 0xC0;
	bResult = MxSetGeneralIoValEx(DEVICE_IO_TYPE,wRSTAddr,dwPortVal,1,SuperIoStruct);

	/* Set time register*/
	dwPortVal = pWdg->refreshTime;
	bResult = MxSetGeneralIoValEx(DEVICE_IO_TYPE,wTimerAddr,dwPortVal,1,SuperIoStruct);

	return 0;
}


/**
 * Close Watchdog
 * @param  [in] fd - the handle
 * @return None
 */
void mxwdg_close(PVOID fd)
{
	PWDG_ST pWdg = (PWDG_ST) fd;
	if (!fd) return;

	DWORD	dwPortVal = 0;
	BOOL	bResult;
	WORD	wRSTAddr = (WORD)WDG_RST_ADDRESS; 
	WORD	wTimerAddr = (WORD)WDG_TIMER_ADDRESS; 
	tagSuperIoStruct SuperIoStruct;

	if(fd != INVALID_HANDLE_VALUE)
	{
		/* Prepare superio parameters */
		SuperIoStruct.bLogicDev = SUPERIO_LOGIC_DEVICE; /*COM1:logic device 1, COM2: logic device 2*/
		SuperIoStruct.bIndexPortAddr = 0x2E;
		SuperIoStruct.bDataPortAddr = 0x2F;
		SuperIoStruct.bIndexAddr = 0x7;  //UCHAR bIndexAddr;		//0x7
		SuperIoStruct.bDelayAddr = 0xEB; //UCHAR bDelayAddr;		//0xEB	
		SuperIoStruct.bIndexPortData = 0x87;//UCHAR bIndexPortData;	//0x87
		SuperIoStruct.bDelayData = 0xAA;	//UCHAR bDelayData;		//0xAA


		/* Turn off WatchDog*/ //dwPortVal  = 0x80;
		bResult = MxGetGeneralIoValEx(DEVICE_IO_TYPE, wRSTAddr, &dwPortVal, 1, SuperIoStruct); //printf("dwPortVal = 0x%x\n",dwPortVal);
		dwPortVal &=  ~KB_RST_VALUE; //printf("dwPortVal  =0x%x\n",dwPortVal);
		bResult = MxSetGeneralIoValEx(DEVICE_IO_TYPE,wRSTAddr,dwPortVal,1,SuperIoStruct);	
	
		/* Shut down WinIo */
		MxGeneralIoClose();
	}

	free(pWdg);
}





#ifndef _DVT
#ifdef __cplusplus    // If used by C++ code, 
}
#endif
#endif
